#ifndef MSP_DATAFILE_PARSER_H_
#define MSP_DATAFILE_PARSER_H_

#include <string>
#include "input.h"

namespace Msp {
namespace DataFile {

class ParserMode;
class Statement;

/**
Frontend for loading datafiles.  Handles switching between text and binary
formats.  A Parser evaluates into a boolean value indicating whether more
statements may be read.
*/
class Parser
{
private:
	Input in;
	std::string main_src;
	std::string src;
	bool good;
	ParserMode *mode;

public:
	Parser(IO::Base &i, const std::string &s);
	~Parser();

	/**
	Reads a statement from the input.  If the end of input was reached, an empty
	invalid statement will be returned.  If an error occurs, the parser will be
	marked as bad and no more statements may be read, even if the exception was
	caught.
	*/
	Statement parse(bool raw = false);

private:
	void process_control_statement(const Statement &);

public:
	operator bool() const { return good && in; }
};

} // namespace DataFile
} // namespace Msp

#endif
