#ifndef MSP_DATAFILE_TYPE_H_
#define MSP_DATAFILE_TYPE_H_

#include <msp/core/inttypes.h>
#include <msp/strings/lexicalcast.h>

namespace Msp {
namespace DataFile {

struct Symbol
{
	std::string name;

	template<typename T>
	Symbol(const T &n): name(lexical_cast<std::string>(n)) { }

	template<typename T> operator T() const { return lexical_cast<T>(name); }
};

struct IntType
{
	static const char signature = 'i';
	typedef Int64 Store;
	typedef Store Load;
};

struct FloatType
{
	static const char signature = 'f';
	typedef double Store;
	typedef Store Load;
};

struct BoolType
{
	static const char signature = 'b';
	typedef bool Store;
	typedef Store Load;
};

struct StringType
{
	static const char signature = 's';
	typedef std::string Store;
	typedef Store Load;
};

struct SymbolType
{
	// For backward compatibility
	static const char signature = 'e';
	typedef Symbol Store;
};

const char valid_signatures[] =
{
	IntType::signature,
	FloatType::signature,
	BoolType::signature,
	StringType::signature,
	SymbolType::signature,
	0
};

template<typename T>
struct HasLoadType
{
	struct Yes { char c[2]; };
	struct No { char c; };

	template<typename U>
	static Yes f(typename U::LoadType *);
	template<typename U>
	static No f(...);

	enum { value = (sizeof(f<T>(0))==sizeof(Yes)) };
};

template<typename T, bool lt = HasLoadType<T>::value>
struct TypeInfo;

template<>
struct TypeInfo<short int, false>: IntType { };

template<>
struct TypeInfo<unsigned short int, false>: IntType { };

template<>
struct TypeInfo<int, false>: IntType { };

template<>
struct TypeInfo<unsigned int, false>: IntType { };

template<>
struct TypeInfo<long int, false>: IntType { };

template<>
struct TypeInfo<unsigned long int, false>: IntType { };

#if defined(_MSC_VER)
template<>
struct TypeInfo<__int64, false>: IntType { };

template<>
struct TypeInfo<unsigned __int64, false>: IntType { };
#elif defined(__GNUC__)
template<>
struct TypeInfo<long long int, false>: IntType { };

template<>
struct TypeInfo<unsigned long long int, false>: IntType { };
#endif

template<>
struct TypeInfo<float, false>: FloatType { };

template<>
struct TypeInfo<double, false>: FloatType { };

template<>
struct TypeInfo<bool, false>: BoolType { };

template<>
struct TypeInfo<std::string, false>: StringType { };

template<>
struct TypeInfo<char *, false>: StringType { };

template<>
struct TypeInfo<const char *, false>: StringType { };

template<typename T>
struct TypeInfo<const T, false>: TypeInfo<T> { };

template<typename T>
struct TypeInfo<T &, false>: TypeInfo<T> { };

template<typename T>
struct TypeInfo<T, true>: TypeInfo<typename T::LoadType>
{ typedef typename T::LoadType Load; };

template<typename T>
struct TypeInfo<T, false>: SymbolType
{ typedef T Load; };

} // namespace DataFile
} // namespace Msp

#endif
